@@ -279,18 +279,21 @@ files are individual messages, C<file> a file with a single message,
 C<mbox> an mbox formatted file, or C<mbx> for an mbx formatted directory.
 
 C<detect> can also be used.  This assumes C<mbox> for any file whose path
-contains the pattern C</\.mbox/i>, C<file> for STDIN and anything that is
-not a directory, or C<directory> otherwise.
+contains the pattern C</\.mbox/i>, C<file> anything that is not a
+directory, or C<directory> otherwise.
 
 =item raw_location
 
-Path to file or directory.  Can be "-" for STDIN.  File globbing is allowed
-using the standard csh-style globbing (see C<perldoc -f glob>).  C<~> at the
-front of the value will be replaced by the C<HOME> environment variable.
-Escaped whitespace is protected as well.
+Path to file or directory.  File globbing is allowed using the
+standard csh-style globbing (see C<perldoc -f glob>).  C<~> at the
+front of the value will be replaced by the C<HOME> environment
+variable.  Escaped whitespace is protected as well.
 
 B<NOTE:> C<~user> is not allowed.
 
+B<NOTE 2:> C<-> is not allowed as a raw location.  To have
+ArchiveIterator deal with STDIN, generate a temp file.
+
 =back
 
 =cut
@@ -727,6 +730,11 @@ sub message_array {
       next;
     }
 
+    if ($rawloc eq '-') {
+      warn 'archive-iterator: raw location "-" is not supported';
+      next;
+    }
+
     # use ham by default, things like "spamassassin" can't specify the type
     $class = substr($class, 0, 1) || 'h';
 
@@ -741,8 +749,7 @@ sub message_array {
           # filename indicates mbox
           $method = \&scan_mailbox;
         } 
-	elsif ($location eq '-' || !(-d $location)) {
-	  # stdin is considered a file if not passed as mbox
+	elsif (!(-d $location)) {
           $method = \&scan_file;
 	}
 	else {
@@ -842,20 +849,34 @@ sub message_array {
 sub mail_open {
   my ($file) = @_;
 
-  my $expr;
+  # bug 5288: the "magic" version of open will strip leading and trailing
+  # whitespace from the expression.  switch to the three-argument version
+  # of open which does not strip whitespace.  see "perldoc -f open" and
+  # "perldoc perlipc" for more information.
+
+  # Assume that the file by default is just a plain file
+  my @expr = ( $file );
+  my $mode = '<';
+
+  # Handle different types of compressed files
   if ($file =~ /\.gz$/) {
-    $expr = "gunzip -cd $file |";
+    $mode = '-|';
+    unshift @expr, 'gunzip', '-cd';
   }
   elsif ($file =~ /\.bz2$/) {
-    $expr = "bzip2 -cd $file |";
+    $mode = '-|';
+    unshift @expr, 'bzip2', '-cd';
   }
-  else {
-    $expr = "$file";
-  }
-  if (!open (INPUT, $expr)) {
+
+  # Go ahead and try to open the file
+  if (!open (INPUT, $mode, @expr)) {
     warn "archive-iterator: unable to open $file: $!\n";
     return 0;
   }
+
+  # bug 5249: mail could have 8-bit data, need this on some platforms
+  binmode INPUT;
+
   return 1;
 }
 
@@ -970,7 +991,7 @@ sub scan_mailbox {
   my ($self, $class, $folder) = @_;
   my @files;
 
-  if ($folder ne '-' && -d $folder) {
+  if (-d $folder) {
     # passed a directory of mboxes
     $folder =~ s/\/\s*$//; #Remove trailing slash, if there
     if (!opendir(DIR, $folder)) {
@@ -1071,7 +1092,7 @@ sub scan_mbx {
   my ($self, $class, $folder) = @_;
   my (@files, $fp);
 
-  if ($folder ne '-' && -d $folder) {
+  if (-d $folder) {
     # got passed a directory full of mbx folders.
     $folder =~ s/\/\s*$//; # remove trailing slash, if there is one
     if (!opendir(DIR, $folder)) {
