@@ -1,9 +1,10 @@
 # <@LICENSE>
-# Copyright 2004 Apache Software Foundation
-# 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to you under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at:
 # 
 #     http://www.apache.org/licenses/LICENSE-2.0
 # 
@@ -384,7 +385,8 @@ sub parse_received_line {
   # Received: (qmail 84907 invoked from network); 13 Feb 2003 20:59:28 -0000
   # Received: (ofmipd 208.31.42.38); 17 Mar 2003 04:09:01 -0000
   # we don't care about this kind of gateway noise
-  if (/^\(/) { return; }
+  # Bug 4943: give /^(from/ a chance to be parsed
+  if (/^\((?!from)/) { return; }
 
   # OK -- given knowledge of most Received header formats,
   # break them down.  We have to do something like this, because
@@ -409,13 +411,17 @@ sub parse_received_line {
     $auth = $1;
   }
   # Sendmail, MDaemon, some webmail servers, and others
-  elsif (/^from .*?(?:\]\)|\)\]) .*?\(.*?authenticated.*?\).*? by/) {
+  elsif (/^from .*?(?:\](?: \([^)]*\))?\)|\)\]) .*?\(.*?authenticated.*?\).*? by/) {
     $auth = 'Sendmail';
   }
   # Critical Path Messaging Server
   elsif (/\) by .+ \(\d{1,2}\.\d\.\d{3}(?:\.\d{1,3})?\) \(authenticated as .+\) id /) {
     $auth = 'CriticalPath';
   }
+  # Postfix 2.3 and later with "smtpd_sasl_authenticated_header yes"
+  elsif (/\)\s+\(Authenticated sender:\s+\S+\)\s+by\s+\S+\s+\(Postfix\)\s+with\s+/) {
+    $auth = 'Postfix';
+  }
 
   if (/^from /) {
     # try to catch enveloper senders
@@ -467,6 +473,14 @@ sub parse_received_line {
       }
     }
 
+    if (/ecelerity /) {
+      if (/^from \(\[(${IP_ADDRESS}):\d+\] helo=(\S+)\) by (\S+) /) {
+        $ip = $1; $helo = $2; $by = $3;
+        / id (\S+) / and $id = $1;
+        goto enough;
+      }
+    }
+
     if (/Exim/) {
       # one of the HUGE number of Exim formats :(
       # This must be scriptable.  (update: it is. cf bug 3950, 3582)
@@ -774,7 +788,7 @@ sub parse_received_line {
     }
 
     # Let's try to support a few qmailish formats in one;
-    # http://bugzilla.spamassassin.org/show_bug.cgi?id=2744#c14 :
+    # http://issues.apache.org/SpamAssassin/show_bug.cgi?id=2744#c14 :
     # Received: from unknown (HELO feux01a-isp) (213.199.4.210) by totor.bouissou.net with SMTP; 1 Nov 2003 07:05:19 -0000 
     # Received: from adsl-207-213-27-129.dsl.lsan03.pacbell.net (HELO merlin.net.au) (Owner50@207.213.27.129) by totor.bouissou.net with SMTP; 10 Nov 2003 06:30:34 -0000 
     if (/^from (\S+) \((?:HELO|EHLO) ([^\)]*)\) \((\S*@)?\[?(${IP_ADDRESS})\]?\).* by (\S+) /)
@@ -790,8 +804,7 @@ sub parse_received_line {
     if (/^from (\S+) \((\S*@)?\[?(${IP_ADDRESS})\]?\).* by (\S+) /)
     {
       $mta_looked_up_dns = 1;
-      # http://bugzilla.spamassassin.org/show_bug.cgi?id=2744 notes that
-      # if HELO == rDNS, qmail drops it.
+      # bug 2744 notes that if HELO == rDNS, qmail drops it.
       $rdns = $1; $helo = $rdns; $ident = (defined $2) ? $2 : '';
       $ip = $3; $by = $4;
       if ($ident) { $ident =~ s/\@$//; }
@@ -877,7 +890,7 @@ sub parse_received_line {
     # Received: from [192.168.1.104] (account nazgul HELO [192.168.1.104])
     # by somewhere.com (CommuniGate Pro SMTP 3.5.7) with ESMTP-TLS id 2088434;
     # Fri, 07 Mar 2003 13:05:06 -0500
-    if (/^from \[(${IP_ADDRESS})\] \(account \S+ (?:HELO|EHLO) (\S*)\) by (\S+) \(/) {
+    if (/^from \[(${IP_ADDRESS})\] \((?:account \S+ )?(?:HELO|EHLO) (\S*)\) by (\S+) \(/) {
       $ip = $1; $helo = $2; $by = $3; goto enough;
     }
 
@@ -1009,6 +1022,15 @@ sub parse_received_line {
       goto enough;
   }
 
+  # Norton AntiVirus Gateway
+  # Received: (from localhost [24.180.47.240])
+  #  by host.name (NAVGW 2.5.2.12) with SMTP id M2006060503484615455
+  #  for <user@domain.co.uk>; Mon, 05 Jun 2006 03:48:47 +0100
+  if (/^\(from (\S*) \[(${IP_ADDRESS})\]\) by (\S+) \(NAVGW .*?\) with /) {
+    $helo = $1; $ip = $2; $by = $3;
+    goto enough;
+  }
+
   # ------------------------------------------------------------------------
   # IGNORED LINES: generally local-to-local or non-TCP/IP handovers
 
@@ -1065,6 +1087,11 @@ sub parse_received_line {
   # (8.12.11/8.12.11/Submit) id iBADHRP6011034; Fri, 10 Dec 2004 13:17:27 GMT
   if (/^from localhost \(localhost \[\[UNIX: localhost\]\]\) by /) { return; }
 
+  # header produced by command line /usr/bin/sendmail -t -f username@example.com
+  # Received: (from username@localhost) by home.example.com
+  # (8.12.11/8.12.11/Submit) id iBADHRP6011034; Fri, 10 Dec 2004 13:17:27 GMT
+  if (/^\(from \S+\@localhost\) by \S+ /) { return; }
+
   # Received: Message by Barricade wilhelm.eyp.ee with ESMTP id h1I7hGU06122 for <spamassassin-talk@lists.sourceforge.net>; Tue, 18 Feb 2003 09:43:16 +0200
   if (/^Message by /) {
     return;	# whatever
