@@ -766,7 +766,7 @@ sub rewrite_report_safe {
 
   foreach my $header (keys %{$self->{conf}->{headers_spam}}) {
     my $data = $self->{conf}->{headers_spam}->{$header};
-    my $line = $self->_process_header($header,$data) || "";
+    my $line = $self->_process_header($header,$data);
     $line = $self->qp_encode_header($line);
     $newmsg .= "X-Spam-$header: $line\n" # add even if empty
   }
@@ -946,7 +946,7 @@ sub rewrite_no_report_safe {
   # use string appends to put this back together -- I finally benchmarked it.
   # join() is 56% of the speed of just using string appends. ;)
   while (my ($header, $data) = each %{$self->{conf}->{$addition}}) {
-    my $line = $self->_process_header($header,$data) || "";
+    my $line = $self->_process_header($header,$data);
     $line = $self->qp_encode_header($line);
     $new_hdrs_pre .= "X-Spam-$header: $line\n";
   }
@@ -2009,6 +2009,7 @@ sub get_uri_detail_list {
   # get URIs from HTML parsing
   # use the metadata version since $self->{html} may not be setup
   my $detail = $self->{msg}->{metadata}->{html}->{uri_detail} || { };
+  $self->{'uri_truncated'} = 1 if $self->{msg}->{metadata}->{html}->{uri_truncated};
 
   # don't keep dereferencing ...
   my $redirector_patterns = $self->{conf}->{redirector_patterns};
@@ -2146,6 +2147,14 @@ sub _get_parsed_uri_list {
       }
     }
 
+    # Make sure all the URIs are nice and short
+    foreach my $uri ( @uris ) {
+      if (length $uri > MAX_URI_LENGTH) {
+        $self->{'uri_truncated'} = 1;
+        $uri = substr $uri, 0, MAX_URI_LENGTH;
+      }
+    }
+
     # setup the cache and return
     $self->{parsed_uri_list} = \@uris;
   }
@@ -2476,7 +2485,7 @@ sub do_meta_tests {
     foreach $token (@tokens) {
 
       # Numbers can't be rule names
-      if ($token =~ /^(?:\W+|\d+)$/) {
+      if ($token =~ /^(?:\W+|[+-]?\d+(?:\.\d+)?)$/) {
         $meta{$rulename} .= "$token ";
       }
       else {
@@ -2487,7 +2496,19 @@ sub do_meta_tests {
           info("rules: meta test $rulename has undefined dependency '$token'");
         }
         elsif ($self->{conf}->{scores}->{$token} == 0) {
-          info("rules: meta test $rulename has dependency '$token' with a zero score");
+          my $conf = $self->{conf};
+          my $dowarn = 1;
+
+          # there are some cases where this is expected; don't warn
+          # in those cases.
+          if ((($self->{conf}->get_score_set()) & 1) == 0 &&
+              $conf->{tflags}->{$token} &&
+              $conf->{tflags}->{$token} =~ /\bnet\b/)
+          {
+            $dowarn = 0;    # bug 5040: net rules in a non-net scoreset
+          }
+
+          $dowarn and info("rules: meta test $rulename has dependency '$token' with a zero score");
         }
 
         # If the token is another meta rule, add it as a dependency
