@@ -1,9 +1,10 @@
 # <@LICENSE>
-# Copyright 2004 Apache Software Foundation
-# 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to you under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at:
 # 
 #     http://www.apache.org/licenses/LICENSE-2.0
 # 
@@ -362,10 +363,16 @@ sub check_dcc {
 
   # short-circuit if there's already a X-DCC header with value of
   # "bulk" from an upstream DCC check
-  if ($permsgstatus->get('ALL') =~ /^X-DCC-(?:[^:]{1,80}-)?Metrics:.*bulk/m) {
+  if ($permsgstatus->get('ALL') =~ /^X-DCC-([^:]{1,80})?-?Metrics:.*bulk/m) {
+    $permsgstatus->{tag_data}->{DCCB} = $1;
+    $permsgstatus->{tag_data}->{DCCR} = "bulk";
     return 1;
   }
 
+  # initialize valid tags
+  $permsgstatus->{tag_data}->{DCCB} = "";
+  $permsgstatus->{tag_data}->{DCCR} = "";
+
   $self->get_dcc_interface();
   return 0 if $self->{dcc_disabled};
 
@@ -630,7 +637,7 @@ sub dccproc_lookup {
 sub plugin_report {
   my ($self, $options) = @_;
 
-  return if $self->{options}->{dont_report_to_dcc};
+  return if $options->{report}->{options}->{dont_report_to_dcc};
   $self->get_dcc_interface();
   return if $self->{dcc_disabled};
 
