@@ -1,12 +1,13 @@
 # <@LICENSE>
-# Copyright 2004 Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to you under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at:
+# 
 #     http://www.apache.org/licenses/LICENSE-2.0
-#
+# 
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -82,11 +83,17 @@ sub extract_metadata {
   }
 
   chop $countries;
-  $msg->{tag_data}{RELAYCOUNTRY} = $countries;
   $msg->put_metadata("X-Relay-Countries", $countries);
   dbg("metadata: X-Relay-Countries: $countries");
 
   return 1;
 }
 
+sub parsed_metadata {
+  my ($self, $opts) = @_;
+  $opts->{permsgstatus}->set_tag ("RELAYCOUNTRY",
+          $opts->{permsgstatus}->get_message->get_metadata('X-Relay-Countries'));
+  return 1;
+}
+
 1;
