@@ -1,11 +1,12 @@
 # spamd prefork scaling, using an Apache-based algorithm
 #
 # <@LICENSE>
-# Copyright 2004 Apache Software Foundation
-# 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to you under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at:
 # 
 #     http://www.apache.org/licenses/LICENSE-2.0
 # 
@@ -121,11 +122,20 @@ sub child_exited {
   $self->compute_lowest_child_pid();
 }
 
+# this is called by SIGTERM and SIGHUP handlers, to ensure that new
+# kids aren't added while the main code is killing the old ones
+# and planning to exit.
+#
+sub set_exiting_flag {
+  my ($self) = @_;
+  $self->{am_exiting} = 1;
+}
+
 sub child_error_kill {
   my ($self, $pid, $sock) = @_;
 
-  warn "prefork: killing failed child $pid ".
-            ($sock ? "fd=".$sock->fileno : "");
+  warn "prefork: killing failed child $pid fd=".
+    ((defined $sock && defined $sock->fileno) ? $sock->fileno : "undefined");
 
   # close the socket and remove the child from our list
   $self->set_child_state ($pid, PFSTATE_KILLED);
@@ -138,7 +148,7 @@ sub child_error_kill {
     $sock->close;
   }
 
-  warn "prefork: killed child $pid";
+  warn "prefork: killed child $pid\n";
 }
 
 sub set_child_state {
@@ -416,7 +426,13 @@ sub order_idle_child_to_accept {
     dbg("prefork: ordered $kid to accept");
 
     # now wait for it to say it's done that
-    return $self->wait_for_child_to_accept($sock);
+    my $ret = $self->wait_for_child_to_accept($kid, $sock);
+    if ($ret) {
+      return $ret;
+    } else {
+      # retry with another child
+      return $self->order_idle_child_to_accept();
+    }
 
   }
   else {
@@ -426,10 +442,11 @@ sub order_idle_child_to_accept {
 }
 
 sub wait_for_child_to_accept {
-  my ($self, $sock) = @_;
+  my ($self, $kid, $sock) = @_;
 
   while (1) {
     my $state = $self->read_one_message_from_child_socket($sock);
+
     if ($state == PFSTATE_BUSY) {
       return 1;     # 1 == success
     }
@@ -437,7 +454,12 @@ sub wait_for_child_to_accept {
       return undef;
     }
     else {
-      die "prefork: ordered child to accept, but child reported state '$state'";
+      warn "prefork: ordered child $kid to accept, but they reported state '$state', killing rogue";
+      $self->child_error_kill($kid, $sock);
+      $self->adapt_num_children();
+      sleep 1;
+
+      return undef;
     }
   }
 }
@@ -469,6 +491,8 @@ sub update_child_status_idle {
 
 sub update_child_status_busy {
   my ($self) = @_;
+
+# if (rand 4 < 1) { $self->report_backchannel_socket("I".pack("N",$self->{pid})."\n");return; warn "TEST for bug 4594"; die; }
   # "B  b1 b2 b3 b4 \n "
   $self->report_backchannel_socket("B".pack("N",$self->{pid})."\n");
 }
@@ -651,6 +675,9 @@ retry_write:
 sub adapt_num_children {
   my ($self) = @_;
 
+  # don't start up new kids while main is working at killing the old ones
+  return if $self->{am_exiting};
+
   my $kids = $self->{kids};
   my $statestr = '';
   my $num_idle = 0;
